import os
import sys
import unittest
sys.path.insert(1, os.path.dirname(os.path.dirname(__file__)))

from api.append import AppendAPI
from utilities.helpers import get_all_keys

class TestAppendAPI(unittest.TestCase):

    api_key = os.environ['tdg_api_key']
    base_url = 'https://api.tdg1.io/v2/append/'
    append_api = AppendAPI(api_key)

    def test_validate_get_phone_results(self):
        expected_keys = [
            'phone', 'phoneType', 'DID', 'recType', 'iDate', 
            'oDate', 'telcoName', 'PHV', 'daCode'
        ]

        result = self.append_api.get_phone(
            first_name="Tom",
            last_name="Smith",
            address="123 Main St",
            zip_number="96162"
        )
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

    def test_validate_get_email_results(self):
        expected_keys = ['email', 'suppression', 'score', 'category']

        result = self.append_api.get_email(
            first_name='',
            last_name='',
            address='656 long lake driv2e',
            zip_number='32765'
        )
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

    def test_validate_get_demographic_results(self):
        expected_keys = [
            'PctBlk', 'PctWht', 'PctHsp', 'PctAsi', 'PctEngSpeak', 'PctSpnSpeak', 'PctAsiaSpeak', 
            'Ethnicity', 'ChildCd', 'ChildAgeCd', 'ChildNbrCd', 'VehLux', 'VehSuv', 'VehTrk', 
            'HHNBRSR', 'CreditCard', 'WealthScr', 'CharityDnr', 'MrktHomeVal', 'Education', 
            'DwellType', 'Married', 'MedYrBld', 'MobHome', 'Pool', 'FirePl', 'SglParent', 
            'PctSFDU', 'PctMFDU', 'MHV', 'MOR', 'PctAuto', 'MedSchl', 'PctWhiteCollar', 
            'Demographics', 'Gender', 'Age', 'DOB', 'LOR', 'Homeowner', 'EHI', 
            'PctBlueCollar', 'PctOccO', 'DemoLvl'
        ]

        result = self.append_api.get_demographic(
            first_name='',
            last_name='',
            address='',
            zip_number='',
            phone='5555555555'
        )
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

    def test_validate_get_auto_results(self):
        expected_keys = [
            'Make', 'Model', 'Year', 'ClassCD', 'FuelTypeCD', 'MFGCD', 'StyleCD', 'Mileages',
            'ClassCD', 'FuelTypeCD', 'MFGCD', 'StyleCD', 'Mileages', 'ODate', 'Vehicle4', 
            'Name', 'FName', 'LName', 'MName', 'Suffix', 'Address', 'House', 'ODate',
            'Year', 'ClassCD', 'FuelTypeCD', 'MFGCD', 'StyleCD', 'Mileages', 'ODate', 
            'State', 'Zip', 'Z4', 'DPC', 'CRTE', 'CNTY', 'Z4Type', 'DPV', 'Vacant', 
            'MSA', 'CBSA', 'IDS', 'PID', 'AID', 'HHID', 'Vehicle', 'Make', 'Model', 
            'PreDir', 'Street', 'StrType', 'PostDir', 'AptType', 'AptNbr', 'City', 
            'Vehicle2', 'Make', 'Model', 'Year', 'ClassCD', 'FuelTypeCD', 'MFGCD', 
            'StyleCD', 'Mileages', 'ODate', 'Vehicle3', 'Make', 'Model', 'Year'
        ]

        result = self.append_api.get_auto(
            full_name='',
            address='',
            zip_number='',
            phone='5555555555'
        )
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

if __name__ == '__main__':
    unittest.main()



import os
import sys
import unittest
sys.path.insert(1, os.path.dirname(os.path.dirname(__file__)))

from api.lookup import LookupAPI
from utilities.helpers import get_all_keys

class TestLookupAPI(unittest.TestCase):

    api_key = os.environ['tdg_api_key']
    base_url = 'https://api.tdg1.io/v2/lookup/'
    lookup_api = LookupAPI(api_key)

    def test_validate_details_by_contact_number_results(self):
        contact_number = 5555555555
        expected_keys = [
            'House', 'PreDir', 'Street', 'StrType', 'PostDir', 'AptType', 'AptNbr', 'City', 
            'ValDate', 'Phone', 'Phone', 'PhoneType', 'DID', 'RecType', 'IDate', 'ODate', 
            'State', 'Zip', 'Z4', 'DPC', 'CRTE', 'CNTY', 'Z4Type', 'DPV', 'Deliverable', 
            'Person', 'Name', 'LName', 'FName', 'MName', 'BusName', 'Address',
            'TelcoName', 'PHV', 'DACode', 'Source'
        ]

        result = self.lookup_api.get_phone(contact_number)
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

    def test_validate_details_by_email_response_results(self):
        email = "abc@gmail.com"
        expected_keys = [
            'Street', 'StrType', 'PostDir', 'AptType', 'AptNbr', 'City', 'State', 'Zip', 
            'Name', 'LName', 'FName', 'MName', 'BusName', 'Address', 'House', 'PreDir', 
            'Z4', 'DPC', 'CRTE', 'CNTY', 'Z4Type', 'DPV', 'Deliverable', 'ValDate', 
            'Email', 'Suppression'
        ]

        result = self.lookup_api.get_email(email)
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

if __name__ == '__main__':
    unittest.main()



import os
import sys
sys.path.insert(1, os.path.dirname(os.path.dirname(__file__)))

from utilities.helpers import get_data

class AppendAPI:
    """
    AppendAPI is a library that interacts with the Append API to fetch details related to phones 
    and emails. To utilize this class, an API key is required for authentication.

    Attributes:
        - BASE_URL (str): The base URL for the Phone Lookup API.
        - headers (dict): Contains the API key in dictionary format.

    Methods:
        - get_phone(first_name: str, 
                        last_name: str, address: str, 
                        zip_number: int) -> dict

        - get_email(first_name: str, 
                        last_name: str, address: str, 
                        zip_number: int) -> dict

        - get_demographic(first_name: str=None, 
                            last_name: str=None, address: str=None, 
                            zip_number: int=None, phone: int=None, 
                            email: str=None, city: str=None, 
                            state: str=None) -> dict

        - get_auto(full_name: str=None, address: str=None, 
                        zip_number: int=None, first_name: str=None, 
                        last_name: str=None, email: str=None, phone: str=None) -> dict:
    
    Example Usage:
        obj = AppendAPI('api_key')
        obj.get_phone_number_details_by_names_and_addresses(
            first_name='first_name',
            last_name='last_name',
            address='address',
            zip_number='zip_number'
        )
    """


    def __init__(self, api_key: str) -> None:
        self.base_url = 'https://api.tdg1.io/v2/append/'
        self.new_base_url = 'https://testapi.tdg1.io/v3/append/'
        self.headers = {
            "x-api-key": api_key
        }

    def get_phone(self, first_name: str,
                        last_name: str, address: str,
                        zip_number: int) -> dict:
        """
        A function that allows users to query for phone numbers matching names and addresses
        by providing certain input fields such as first name, last name, address, and zip 
        code. The API then returns appended telephone numbers along with other phone 
        attributes such as record type, telephone confidence score, and directory assistance flag.

        Parameters:
            - first_name: str, (REQUIRED)
                The first name of the individual (up to 15 characters).
            - last_name: str, (REQUIRED)
                The last name of the individual (up to 15 characters).
            - address: str, (REQUIRED)
                The address of the individual (up to 60 characters).
            - zip_number: int, (REQUIRED)
                5-digit numeric USPS zip code. Either City/State or Zip is required.
        Returns:
            dict
                A dictionary containing the details of the retrieved response.
        """

        return get_data(
            base_url=self.new_base_url,
            request_type='POST',
            endpoint='phone',
            payload={
                "FName": first_name,
                "LName": last_name,
                "Address1": address,
                "Zip": zip_number
            },
            headers=self.headers
        )

    def get_email(self, first_name: str,
                                last_name: str, address: str,
                                zip_number: int) -> dict:
        """
        A function that retrieves email details based on names and addresses.
        The Api returns appended email when suppression is False along with other attributes 
        score, category, suppression

        Parameters:
            - first_name: str, (REQUIRED)
                The first name of the individual (up to 15 characters).
            - last_name: str, (REQUIRED)
                The last name of the individual (up to 20 characters).
            - address: str, (REQUIRED)
                The address of the individual (up to 64 characters).
            - zip_number: int, (REQUIRED)
                5-digit numeric USPS zip code.

        Returns:
            dict
                A dictionary containing the details of the retrieved response.
        """

        return get_data(
            base_url=self.new_base_url,
            request_type='POST',
            endpoint='email',
            payload={
                "FName": first_name,
                "LName": last_name,
                "Address1": address,
                "Zip": zip_number
            },
            headers=self.headers
        )

    def get_demographic(self, first_name: str,
                                    last_name: str, address: str, zip_number: int,
                                    phone: int=None, email: str=None, city: str=None,
                                    state: str=None):
        """      
        A function that retrieves details demographic details based on 
        names, addresses, (email or phone number).

        Parameters:
            - first_name: str, (Should be a combination of FName, LName, Zip, and 
                Address parameters; Phone or Email is REQUIRED)
                The first name of the individual (up to 15 characters).
            - last_name: str, (Should be a combination of FName, LName, Zip, and 
                Address parameters; Phone or Email is REQUIRED)
                The last name of the individual (up to 20 characters).            
            - address: str, (Should be a combination of FName, LName, Zip, and 
                Address parameters; Phone or Email is REQUIRED)
                Address 64 characters max.
            - zip_number: int, (Should be a combination of FName, LName, Zip, and 
                Address parameters; Phone or Email is REQUIRED)
                5-digit numeric USPS zip code. * Either City/State or Zip is required.
            - phone: int, (REQUIRED if Email is not included and other parameters are included)
                10 digit numeric phone number (without spaces, dashes, or parentheses)
            - email: str, (REQUIRED if phone is not included and other parameters are included)
                Email address, 100 characters max
            - city: (OPTIONAL)
                City name, 28 characters max. * Either City/State or Zip is required.
            - state: (OPTIONAL)
                2 character state abbreviation. * Either City/State or Zip is required.

        Returns:
            dict
                A dictionary containing the details of the retrieved response.

        Raises:
            ValueError: either one of phone or email is missing.
        """

        if not phone and not email:
            raise ValueError("Either of phone or email is required")

        return get_data(
            base_url=self.base_url,
            request_type='POST',
            endpoint='demographic',
            payload={
                "FName": first_name,
                "LName": last_name,
                "Address1": address,
                "Zip": zip_number,
                "Phone": phone,
                "Email": email,
                "City": city,
                "State": state
            },
            headers=self.headers
        )

    def get_auto(self, full_name: str, address: str,
                    zip_number: int, email: str=None, phone: str=None) -> dict:
        """
        A function that retrieves Auto details based on names and addresses.

        Parameters:
            - full_name: str, (REQUIRED)
                The full name of the individual
            - address: str, (REQUIRED)
                The address of the individual (up to 64 characters).
            - zip_number: int, (REQUIRED)
                5-digit numeric USPS zip code.
            - phone: int, (REQUIRED if Email is not included and other parameters are included)
                10 digit numeric phone number (without spaces, dashes, or parentheses)
            - email: str, (REQUIRED if phone is not included and other parameters are included)
                Email address, 100 characters max
        Returns:
            dict
                A dictionary containing the details of the retrieved response.
        """

        return get_data(
            base_url=self.base_url,
            request_type='POST',
            endpoint='auto',
            payload={
                "Address1": address,
                "Zip": zip_number,
                "FullName": full_name,
                "Phone": phone,
                "Email": email
            },
            headers=self.headers
        )



import json
import requests
from urllib3.util import Retry
from requests.adapters import HTTPAdapter
import logging

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def get_data(base_url: str, endpoint: str, headers: dict, request_type: str = 'GET', params: dict = {}, payload: dict = {}) -> dict:
    """
    A function that extracts data uniformly for all endpoints, with custom
    configuration passed to them.

    Parameters:
    - endpoint (str): The endpoint to send the request to.
    - request_type (str): The type of request to send (default is 'GET').
    - params (dict): Additional parameters to include in the request 
      (default is an empty dictionary).
    - payload (dict): Data payload to send with the request (default is an empty dictionary).

    Attributes:
    - BASE_URL: A class variable to store the base URL.
    - headers (dict): Stores the API key in the form of a dictionary.
    
    Returns:
    - dict: The response data received from the request.
    """

    url = ''.join([base_url, endpoint])
    session = requests.Session()
    retries = Retry(total=3,
                    backoff_factor=0.1,
                    status_forcelist=[502, 503, 504, 404, 400, 429])
    adapter = HTTPAdapter(max_retries=retries)
    session.mount('https://', adapter)
    session.mount('http://', adapter)
    try:
        response = session.request(
            url=url,
            method=request_type,
            headers=headers,
            params=params,
            data=json.dumps(payload)
        )

        logger.info(f"Request sent to {url} with status code {response.status_code}")

        try:
            response.raise_for_status()
        except Exception:
            logger.error(f"Error: {response.text}")
            return eval(response.text)['message']

        if response.status_code == 204:
            logger.info('No Content')
            return {}
        return response.json()

    except requests.exceptions.ConnectionError:
        logger.error("Error Occured, You're not connected with internet")
        raise Exception("Error Occured, You're not connected with internet")


def get_all_keys(data):
    """
    Recursively iterates through a nested dictionary to retrieve all keys.
    """
    keys = []
    if isinstance(data, dict):
        for key, value in data.items():
            keys.append(key)
            keys.extend(get_all_keys(value))
    keys.sort()
    return keys




import os
import sys
sys.path.insert(1, os.path.dirname(os.path.dirname(__file__)))

from utilities.helpers import get_data

class LookupAPI:
    """
    LookupAPI is a library that interacts with the Phone Lookup API to fetch details related to 
    phone numbers and emails. To utilize this class, an API key is required for authentication.

    Attributes:
        - BASE_URL (str): The base URL for the Phone Lookup API.
        - headers (dict): Contains the API key in dictionary format.

    Methods:
        - get_phone(phone_number: int) -> dict:
            Retrieves details for a single phone  number.

        - get_phone_number_details_in_bulk(phone_numbers_list: list) -> list:
            Retrieves details for multiple phone  numbers in bulk.

        - get_email(email) -> dict:
            Retrieves details for a single email.

        - get_email_details_in_bulk(email_list) -> lis:
            Retrieves details for multiple emails.
    
    Example Usage:
        obj = LookupAPI('api_key')
        obj.get_details_by_phone_number(phone_number=5555555555)
    """

    def __init__(self, api_key: str) -> None:
        self.base_url = 'https://testapi.tdg1.io/v3/lookup/'
        self.headers = {
            "x-api-key": api_key
        }

    def get_phone(self, phone_number: int) -> dict:

        """
        A function that gets the details for a single phone number.
        
        Parameters:
            - phone_number: int, (REQUIRED)
                10-digit numeric phone number (without spaces, dashes, or parentheses).

        Returns:
            A dictionary containing the details of the phone  number.

        Raises:
            ValueError: If phone_number is not a 10-digit number.
        """
        if len(str(phone_number)) != 10:
            len(str(phone_number))
            raise ValueError("Contact number must be a 10-digit numeric phone number.")


        return get_data(
            base_url=self.base_url,
            request_type='POST',
            endpoint='phone',
            payload={
                "Phone": phone_number
            },
            headers=self.headers
        )
    

    def get_phone_details_in_bulk(self, phone_numbers_list: list) -> list:
        """
        A function that gets the details for multiple phone  numbers in bulk.
        
        Parameters:
            - phone_numbers: list, (REQUIRED)
                List of 10-digit numeric phone numbers (without spaces, dashes, or parentheses), 
                not greater than 100 numbers in a list

        Returns:
            A list of dictionaries containing the details of the phone  numbers.
        
        Raises:
            ValueError: If any phone  number in the list is not a 10-digit number.
        """
        if len(phone_numbers_list) > 100:
            raise ValueError("Maximum range of allowed numbers are 100.")

        for phone_number in phone_numbers_list:
            if len(str(phone_number)) != 10:
                raise ValueError(f"phone number: {phone_number} is not a 10 digit number, \
                All phone  numbers in the list must be 10-digit numeric phone numbers.")

        output_data = [
                        get_data(
                            base_url=self.base_url,
                            headers=self.headers,
                            request_type='POST',
                            endpoint='phone',
                            payload={
                                "Phone": phone 
                            }
                        ) for phone in phone_numbers_list
                    ]

        return output_data

    def get_email(self, email: str) -> dict:
        """
        A function that gets the details by email.
        
        Parameters:
            - email: str, (REQUIRED)
                100 characters max.

        Returns:
            A dictionary containing the details of the email.

        Raises:
            ValueError: If email is not a 10-digit number.
        """

        if '@' not in email:
            raise ValueError("Invalid email address.")

        return get_data(
            base_url=self.base_url,
            request_type='POST',
            endpoint='email',
            payload={
                "Email": email
            },
            headers=self.headers
        )

    def get_email_details_in_bulk(self, email_list: list) -> list:
        """
        A function that gets the details for multiple emails in bulk.
        
        Parameters:
            - email_list: list, (REQUIRED)
                list of valid emails not greater than 100 emails in a list
        Returns:
            A list of dictionaries containing the details of the phone numbers.
        
        Raises:
            ValueError: If any phone number in the list is not a 10-digit number.
        """
        if len(email_list) > 100:
            raise ValueError("Maximum range of allowed emails are 100 .")

        for email in email_list:
            if '@' not in email:
                raise ValueError(f"email: {email} is not valid.")

        output_data = [
                        get_data(
                            base_url=self.base_url,
                            headers=self.headers,
                            request_type='POST',
                            endpoint='email',
                            payload={
                                "Email": email
                            }
                        ) for email in email_list
                    ]

        return output_data



import os
import sys
import unittest
sys.path.insert(1, os.path.dirname(os.path.dirname(__file__)))

from api.verify import VerifyAPI
from utilities.helpers import get_all_keys

class TestVerifyAPI(unittest.TestCase):

    api_key = os.environ['tdg_api_key']
    base_url = 'https://api.tdg1.io/v2/verify/'
    verify_api = VerifyAPI(api_key)

    def test_validation_verification_score_results(self):
        phone = 5555555555
        expected_keys = [
            'RiskFlagCount', 'ValidationSummary', 'LinkageSummary', 'RiskFlagSummary',
            'IDScores', 'ValidAddress', 'ValidPhone', 'ValidEmail', 'ValidName', 'ValidZip', 
            'Phone2ConfidenceScore', 'AddressConfidenceScore', 'ValidCount', 'LinkageCount', 
            'Deceased', 'NameToPhone', 'NameToEmail', 'NameToAddress', 'AddressToPhone', 
            'AddressToEmail', 'PhoneToEmail', 'ZipToPhone', 'USLocation', 'ValidPhone2', 'IDVerifyScore',
            'NameToPhone2', 'AddressToPhone2', 'Phone2ToEmail', 'PhoneConfidenceScore', 
            ]

        result = self.verify_api.get_id(phone=phone)
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

    def test_validation_verify_tcpa_info_results(self):
        phone=5555555555
        last_name='Smith'
        expected_keys = ["verification_code", "phone_type"]

        result = self.verify_api.get_tcpa_info(phone=phone, last_name=last_name)
        keys_extracted = get_all_keys(result)
        expected_keys.sort()
        self.assertEqual(keys_extracted, expected_keys)

if __name__ == '__main__':
    unittest.main()



import os
import sys
sys.path.insert(1, os.path.dirname(os.path.dirname(__file__)))

from utilities.helpers import get_data

class VerifyAPI:
    """
    VerifyApi is a library that interacts with the Verify API to verify the validity of 
    consumer contact information and offering tcpa identity verification. 
    To utilize this class, an API key is required for authentication.

    Attributes:
        - BASE_URL (str): The base URL for the Phone Lookup API.
        - headers (dict): Contains the API key in dictionary format.

    Methods:
        - get_id(first_name: str=None, last_name: str=None, 
                    address: int=None, zip_number: int=None, 
                    phone: str=None, email: str=None) -> dict

        - get_tcpa_info(self, phone: int=None, last_name: str=None) -> dict
    
    Example Usage:
        obj = VerifyAPI('api_key')
        obj.get_verification_score(first_name='TOM', last_name='SMITH')
    """

    def __init__(self, api_key: str) -> None:
        self.base_url = 'https://api.tdg1.io/v2/verify/'
        self.headers = {
            "x-api-key": api_key
        }

    def get_id(self, first_name: str=None, last_name: str=None,
                    address: int=None, zip_number: int=None,
                    phone: str=None, email: str=None) -> dict:
        """
        This function is used to assess the validity of consumer contact information, 
        identify potential risks or fraud, and make informed decisions based on the 
        verification scores and summaries provided by the service.

        Parameters(ATLEAST 2 PARAMETERS WITH ANY COMBINATION REQUIRED TO CHECK THE ACCURACY SCORE):
            - first_name: str
                The first name of the individual. Maximum 15 characters.
            - last_name: str
                The last name of the individual. Maximum 20 characters.
            - address: str
                The address of the individual. Maximum 64 characters.
            - zip_number: int
                The 5-digit numeric USPS zip code. Either City/State or Zip is required.
            - phone: str
                The 10-digit numeric phone number without spaces, dashes, or parentheses.
            - email: str
                The email address of the individual. Maximum 100 characters.

        Returns:
            dict
                A dictionary containing the details of the retrieved response.
        """

        return get_data(
            base_url=self.base_url,
            request_type='POST',
            endpoint='id',
            payload={
                "FName": first_name,
                "LName": last_name,
                "Address1": address,
                "Zip": zip_number,
                "Phone": phone,
                "Email": email
            },
            headers=self.headers
        )

    def get_tcpa_info(self, phone: int=None, last_name: str=None) -> dict:
        """
        function to make a request to the TCPA Verify endpoint to mitigate 
        TCPA risk by identifying wireless numbers and offering identity verification 
        for called party consent.

        Parameters:
            - phone (Required)  
                10 digit numeric phone number (without spaces, dashes, or parentheses)  
            
            - last_name (Required)
                The last name of the individual. Maximum 20 characters.

        Returns:
            dict
                A dictionary containing the details of the retrieved response.
        """

        return get_data(
            base_url=self.base_url,
            request_type='POST',
            endpoint='tcpa',
            payload={
                "phone": phone,
                "last_name": last_name
            },
            headers=self.headers
        )

def get_email(self, email: str) -> dict:
    """
    A function that gets the details by email.

    Parameters:
        - email: str, (REQUIRED)
            100 characters max.

    Returns:
        A dictionary containing the details of the email.

    Raises:
        ValueError: If email is not valid.
    """

    if '@' not in email:
        raise ValueError("Invalid email address.")

    return get_data(
        base_url=self.base_url,
        request_type='POST',
        endpoint='email',
        payload={
            "Email": email
        },
        headers=self.headers
    )
